import { j as getVideo, p as postHistory, d as getPersonalPlaylists, l as getComments, m as getDislikes, n as amSubscribed } from "../../../../../chunks/index3.js";
import { f as playerProxyVideosStore, r as returnYtDislikesStore, b as authStore, h as returnYTDislikesInstanceStore } from "../../../../../chunks/store.js";
import { d as decodeHtmlCharCodes } from "../../../../../chunks/misc.js";
import { a as convertToSeconds } from "../../../../../chunks/numbers.js";
import { e as error } from "../../../../../chunks/index4.js";
import { g as get } from "../../../../../chunks/index.js";
function extractActualLink(url) {
  const urlParams = new URLSearchParams(url.split("?")[1]);
  const actualLink = urlParams.get("q");
  if (actualLink) {
    return decodeURIComponent(actualLink);
  }
  return url;
}
function processYoutubeLink(line) {
  const urlRegex = /<a href="https:\/\/www\.youtube\.com\/redirect\?([^"]+)"/;
  const urlMatch = urlRegex.exec(line);
  if (urlMatch) {
    const redirectUrl = urlMatch[0];
    const actualUrl = extractActualLink(redirectUrl);
    return line.replace(urlRegex, `<a href="${actualUrl}"`);
  } else {
    return line;
  }
}
function phaseDescription(videoId, content, fallbackPatch) {
  const timestamps = [];
  const lines = content.split("\n");
  const urlRegex = /<a href="([^"]+)"/;
  const timestampRegexInvidious = /<a href="([^"]+)" data-onclick="jump_to_time" data-jump-time="(\d+)">(\d+:\d+(?::\d+)?)<\/a>\s*(.+)/;
  const timestampRegexYtJs = new RegExp(
    `href="https://www\\.youtube\\.com/watch\\?v=${videoId}(?:&t=(\\d+)s)?"[^>]*>\\s*<span[^>]*>\\s*([^<]+)\\s*</span>\\s*</a>\\s*<span[^>]*>\\s*([^<]+)\\s*</span>`,
    "i"
  );
  let filteredLines = [];
  lines.forEach((line) => {
    const urlMatch = urlRegex.exec(line);
    const timestampMatch = (fallbackPatch === "youtubejs" ? timestampRegexYtJs : timestampRegexInvidious).exec(fallbackPatch === "youtubejs" ? line + "</span>" : line);
    if (urlMatch !== null && timestampMatch === null) {
      const modifiedLine = processYoutubeLink(line).replace(
        /<a href="([^"]+)"/,
        '<a href="$1" target="_blank" rel="noopener noreferrer" class="link"'
      );
      console.log(modifiedLine);
      filteredLines.push(modifiedLine);
    } else if (timestampMatch !== null) {
      const time = (fallbackPatch === "youtubejs" ? timestampMatch[1] : timestampMatch[2]) || "0";
      const timestamp = fallbackPatch === "youtubejs" ? timestampMatch[2] : timestampMatch[3];
      const title = fallbackPatch === "youtubejs" ? timestampMatch[3] || "" : timestampMatch[4] || "";
      timestamps.push({
        time: convertToSeconds(time),
        // Remove any HTML in the timestamp title.
        title: decodeHtmlCharCodes(title.replace(/<[^>]+>/g, "").replace(/\n/g, "").trim()),
        timePretty: timestamp
      });
    } else {
      filteredLines.push(line);
    }
  });
  const filteredContent = filteredLines.join("\n");
  return { description: filteredContent, timestamps };
}
async function load({ params, url }) {
  let video;
  try {
    video = await getVideo(params.slug, get(playerProxyVideosStore), { priority: "high" });
  } catch (errorMessage) {
    error(500, errorMessage);
  }
  let personalPlaylists;
  if (get(authStore)) {
    postHistory(video.videoId);
    personalPlaylists = getPersonalPlaylists({ priority: "low" });
  } else {
    personalPlaylists = null;
  }
  let comments;
  try {
    comments = video.liveNow ? null : getComments(params.slug, { sort_by: "top", source: "youtube" }, { priority: "low" });
  } catch {
    comments = null;
  }
  let returnYTDislikes;
  const returnYTDislikesInstance = get(returnYTDislikesInstanceStore);
  if (returnYTDislikesInstance && returnYTDislikesInstance !== "") {
    try {
      returnYTDislikes = get(returnYtDislikesStore) ? getDislikes(params.slug, { priority: "low" }) : null;
    } catch {
    }
  }
  return {
    video,
    content: phaseDescription(video.videoId, video.descriptionHtml, video.fallbackPatch),
    playlistId: url.searchParams.get("playlist"),
    streamed: {
      personalPlaylists,
      returnYTDislikes,
      comments,
      subscribed: amSubscribed(video.authorId)
    }
  };
}
export {
  load
};
